// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file CNC.ino
// Main application entry point for the CNC gcode interpreter and driver.
// 
// @brief CNC gcode interpreter for 3 axis CNC machine. Configured for ELFSHL1
//     3 axis stepper motor driver shield.
//
// Acknowledgements:
// Inspired on the work of Carlos guilarte y diego colonnello (Txapuzas CNC).
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#if (ARDUINO <  100)
#include <WProgram.h>
#else
#include <Arduino.h>
#endif

#include <Wire.h> 

#include "configMe.h"
#include "processGcode.h"
#include "tempCntrl.h"
#include "mmi.h"

#ifdef _MMI_ENABLED_
#include <LiquidCrystal_I2C.h>
#endif //_MMI_ENABLED_

#ifdef _TEMP_CONTROL_ENABLED_
#include <thermistor.h>
#include <lm75.h>
#endif // _TEMP_CONTROL_ENABLED_


// Variable and object declarations
// ---------------------------------------------------------------------------

#ifdef _TEMP_CONTROL_ENABLED_
tempCntrl         myTempControl(FAN_PIN, 0b1001000);
#endif // _TEMP_CONTROL_ENABLED_

#ifdef _MMI_ENABLED_
LiquidCrystal_I2C lcd(0x38);      // Set the LCD I2C address
mmi               myMMI;
#endif // _MMI_ENABLED_

stepperCntrl      machController; // machine controller


// Initialize the system and electronics
// ----------------------------------------------------------------------------
void setup()
{
   // Initialise all the electronics
   Serial.begin(19200);
   
#ifdef _TEMP_CONTROL_ENABLED_   
   myTempControl.init();
#endif // _TEMP_CONTROL_ENABLED_
   
   machController.init ();
   
   //other initialization.
   initGcodeProcessor( &machController );
   
#ifdef _MMI_ENABLED_   
   // initialize the lcd
   lcd.begin(16,2);                    
   
   // initialize the LCD backlight pin
   pinMode ( LCD_BL_PIN, OUTPUT ); 
   digitalWrite ( LCD_BL_PIN, HIGH );
   
   // initialize the MMI, with or without thermal control
#ifdef _TEMP_CONTROL_ENABLED_
   myMMI.init ( &lcd, &myTempControl, &machController );
#else
   myMMI.init ( &lcd, &machController );
#endif // _TEMP_CONTROL_ENABLED_
   
#endif // _MMI_ENABLED_
}


// Main application control thread
// ----------------------------------------------------------------------------
void loop()
{
   static int counter = 0;
   
   if ( counter == 0 )
   {
      counter=10000;
#ifdef _TEMP_CONTROL_ENABLED_
      myTempControl.controlTemp();
#endif //_TEMP_CONTROL_ENABLED_
      
#ifdef _MMI_ENABLED_   
      myMMI.display ();
#endif // _MMI_ENABLED_
   }
   counter--;
   
   processGcode ();
}
